@@ -18,8 +18,13 @@ from hapless.utils import kill_proc_tree, logger, wait_created
 
 
 class Hapless:
-    def __init__(self, hapless_dir: Optional[Union[Path, str]] = None):
-        self.ui = ConsoleUI()
+    def __init__(
+        self,
+        hapless_dir: Optional[Union[Path, str]] = None,
+        *,
+        quiet: bool = False,
+    ):
+        self.ui = ConsoleUI(disable=quiet)
         user = getpass.getuser()
         default_dir = Path(tempfile.gettempdir()) / "hapless"
 
@@ -110,16 +115,22 @@ class Hapless:
         cmd: str,
         hid: Optional[str] = None,
         name: Optional[str] = None,
+        *,
+        redirect_stderr: Optional[bool] = None,
     ) -> Hap:
         hid = hid or self._get_next_hap_id()
         hap_dir = self._hapless_dir / f"{hid}"
         hap_dir.mkdir()
-        return Hap(hap_dir, cmd=cmd, name=name)
+        if redirect_stderr is None:
+            redirect_stderr = config.REDIRECT_STDERR
+        return Hap(hap_dir, cmd=cmd, name=name, redirect_stderr=redirect_stderr)
 
-    def _run_hap_subprocess(self, hap: Hap):
-        with open(hap.stdout_path, "w") as stdout_pipe, open(
-            hap.stderr_path, "w"
-        ) as stderr_pipe:
+    def _wrap_subprocess(self, hap: Hap):
+        try:
+            stdout_pipe = open(hap.stdout_path, "w")
+            stderr_pipe = stdout_pipe
+            if not hap.redirect_stderr:
+                stderr_pipe = open(hap.stderr_path, "w")
             self.ui.print(f"{config.ICON_INFO} Launching", hap)
             shell_exec = os.getenv("SHELL")
             if shell_exec is not None:
@@ -137,10 +148,13 @@ class Hapless:
             hap.bind(pid)
 
             retcode = proc.wait()
+        finally:
+            stdout_pipe.close()
+            stderr_pipe.close()
 
-            # TODO: accessing private attribute, should be `set_rc` method
-            with open(hap._rc_file, "w") as rc_file:
-                rc_file.write(f"{retcode}")
+        # TODO: accessing private attribute, should be `set_rc` method
+        with open(hap._rc_file, "w") as rc_file:
+            rc_file.write(f"{retcode}")
 
     def _check_fast_failure(self, hap: Hap):
         if wait_created(hap._rc_file) and hap.rc != 0:
@@ -171,18 +185,55 @@ class Hapless:
         self,
         hap: Hap,
         check: bool = False,
+        *,
+        blocking: bool = False,
     ) -> None:
         """
         Run hap in a separate process.
         If `check` is True, it will check for fast failure and exit
         if hap terminates too quickly.
         """
+        if blocking:
+            # NOTE: this is for the testing purposes only
+            self._wrap_subprocess(hap)
+            return
+
+        # TODO: or sys.platform == "win32"
+        if config.NO_FORK:
+            logger.debug("Forking is disabled, running using spawn via wrapper")
+            self._run_via_spawn(hap)
+        else:
+            logger.debug("Running hap using fork")
+            self._run_via_fork(hap)
+
+        logger.debug(f"Parent process continues with pid {os.getpid()}")
+        if check:
+            self._check_fast_failure(hap)
+
+    def _run_via_spawn(self, hap: Hap) -> None:
+        exec_path = shutil.which("hapwrap")
+        if exec_path is None:
+            self.ui.error(
+                "Cannot find wrapper to run process. Please reinstall hapless"
+            )
+            sys.exit(1)
+
+        proc = subprocess.Popen(
+            [f"{exec_path}", f"{hap.hid}"],
+            start_new_session=True,
+            stdin=subprocess.DEVNULL,
+            stdout=subprocess.DEVNULL,
+            stderr=subprocess.DEVNULL,
+        )
+        logger.debug(f"Running subprocess in child with pid {proc.pid}")
+        logger.debug(f"Using wrapper located at {exec_path}")
+
+    def _run_via_fork(self, hap: Hap) -> None:
         pid = os.fork()
         if pid == 0:
-            self._run_hap_subprocess(hap)
-        else:
-            if check:
-                self._check_fast_failure(hap)
+            logger.debug(f"Running subprocess in child with pid {os.getpid()}")
+            self._wrap_subprocess(hap)
+            # NOTE: sole purpose of the child is to run a subprocess
             sys.exit(0)
 
     def run_command(
@@ -191,13 +242,18 @@ class Hapless:
         hid: Optional[str] = None,
         name: Optional[str] = None,
         check: bool = False,
+        *,
+        redirect_stderr: Optional[bool] = None,
+        blocking: bool = False,
     ) -> None:
         """
         For the command provided create a hap and run it.
         If `hid` or `name` is not provided, it will be generated automatically.
         """
-        hap = self.create_hap(cmd=cmd, hid=hid, name=name)
-        self.run_hap(hap, check=check)
+        hap = self.create_hap(
+            cmd=cmd, hid=hid, name=name, redirect_stderr=redirect_stderr
+        )
+        self.run_hap(hap, check=check, blocking=blocking)
 
     def logs(self, hap: Hap, stderr: bool = False, follow: bool = False):
         filepath = hap.stderr_path if stderr else hap.stdout_path
@@ -207,14 +263,23 @@ class Hapless:
                 style=f"{config.COLOR_MAIN} bold",
             )
             return subprocess.run(["tail", "-f", filepath])
-        else:
-            return subprocess.run(["cat", filepath])
+
+        text = filepath.read_text()
+        if not text:
+            self.ui.error("No logs found")
+            return
+
+        self.ui.print(
+            f"{config.ICON_INFO} Showing logs at {filepath}",
+            style=f"{config.COLOR_MAIN} bold",
+        )
+        self.ui.print_plain(text)
 
     def _clean_haps(self, filter_haps) -> int:
         haps = list(filter(filter_haps, self.get_haps()))
         for hap in haps:
             logger.debug(f"Removing {hap.path}")
-            shutil.rmtree(hap.path)
+            shutil.rmtree(hap.path, ignore_errors=True)
         return len(haps)
 
     def _clean_one(self, hap: Hap):
@@ -266,9 +331,14 @@ class Hapless:
         else:
             self.ui.error("Cannot send signal to the inactive hap")
 
-    def restart(self, hap: Hap):
-        hid, name, cmd, restarts = hap.hid, hap.name, hap.cmd, hap.restarts
-
+    def restart(self, hap: Hap) -> None:
+        hid, name, cmd, restarts, redirect_stderr = (
+            hap.hid,
+            hap.name,
+            hap.cmd,
+            hap.restarts,
+            hap.redirect_stderr,
+        )
         if hap.active:
             self.kill([hap], verbose=False)
 
@@ -276,10 +346,17 @@ class Hapless:
         while hap_killed.active:
             hap_killed = self.get_hap(hid)
 
+        rc_exists = wait_created(hap_killed._rc_file, timeout=1)
+        if not rc_exists:
+            logger.error(
+                f"Hap {hap_killed} process was killed, "
+                f"but parent did not write return code"
+            )
+
         self._clean_one(hap_killed)
 
         name = f"{name}{config.RESTART_DELIM}{restarts + 1}"
-        self.run_command(cmd=cmd, hid=hid, name=name)
+        self.run_command(cmd=cmd, hid=hid, name=name, redirect_stderr=redirect_stderr)
 
     def rename_hap(self, hap: Hap, new_name: str):
         rich_text = (
