@@ -8,13 +8,25 @@ from hapless.hap import Hap
 
 
 class ConsoleUI:
-    def __init__(self):
-        self.console = Console(highlight=False)
+    def __init__(self, disable: bool = False) -> None:
+        self.console = Console(highlight=False, quiet=disable)
         self.default_formatter = TableFormatter()
+        self.disable = disable
 
     def print(self, *args, **kwargs):
         return self.console.print(*args, **kwargs)
 
+    def print_plain(self, text: str):
+        return self.console.print(
+            text,
+            markup=False,
+            highlight=False,
+            emoji=False,
+            no_wrap=True,
+            overflow="ignore",
+            crop=False,
+        )
+
     def error(self, message: str):
         return self.console.print(
             f"{config.ICON_INFO} {message}",
